--[[
	Scripts that add the "kommunal" mission type and loads the associated vehicles
	-- Authors: SMI Modding Team (Nicola)
    -- Game version: Farming Simulator 22
    -- Version: 1.0 - 3.07.2024	
]]


KommunalMission = {
	debug = false,
	REWARD_PER_HA = 500,
	SUCCESS_FACTOR = 0.90,
	REWARD_PER_METER = 5,
	mission_vehicles_xml = g_currentModName .. "/missionVehicles/vehicles.xml",
}

local modName = g_currentModName

local KommunalMission_mt = Class(KommunalMission, WeedMission)
InitObjectClass(KommunalMission, "KommunalMission")

function KommunalMission.new(isServer, isClient, customMt)
	local self = WeedMission.new(isServer, isClient, customMt or KommunalMission_mt)
	self.workAreaTypes = {
		[WorkAreaType.MULCHER] = true,
		[WorkAreaType.MOWER] = true
	}
	--self.rewardPerHa = KommunalMission.REWARD_PER_HA
	self.area_trimmed = 0
	self.reimbursementPerHa = 0
	self.area_tot = math.random()*500 + 1000
	self.rewardPerHa = KommunalMission.REWARD_PER_HA * self.area_tot/1250
	return self
end

-- TODO: vedere se il caricamento funziona bene
function KommunalMission:loadFromXMLFile(xmlFile, key)
	-- se non ci sono le funzionalità di base
	-- se non ci sono le funzionalità di base
	if not KommunalMission:superClass().loadFromXMLFile(self, xmlFile, key) then
		return false
	end

	local field_key = key .. ".field"
	self.area_tot = getXMLFloat(xmlFile, field_key .. "#areaToCut")
	self.area_trimmed = getXMLFloat(xmlFile, field_key .. "#areaTrimmed")
	return true
end

function KommunalMission:saveToXMLFile(xmlFile, key) 
	setXMLString(xmlFile, key .. "#type", "kommunal")
	setXMLInt(xmlFile, key .. "#reward", self.reward or 100)
	setXMLInt(xmlFile, key .. "#status", self.status)	 -- da capire
	setXMLBool(xmlFile, key .. "#success", self.area_trimmed/self.area_tot >= 1)	 -- da capire

	local field_key = key .. ".field"
	setXMLInt(xmlFile, field_key .. "#id", self.field.fieldId)
	setXMLFloat(xmlFile, field_key .. "#sprayFactor", self.sprayFactor)
	setXMLBool(xmlFile, field_key .. "#spraySet", false)
	setXMLFloat(xmlFile, field_key .. "#plowFactor", self.fieldPlowFactor)

	setXMLInt(xmlFile, field_key .. "#state", 2)
	setXMLInt(xmlFile, field_key .. "#vehicleGroup", self.vehicleGroupIdentifier)
	setXMLFloat(xmlFile, field_key .. "#vehicleUseCost", self.vehicleUseCost)

	setXMLInt(xmlFile, field_key .. "#growthState", 6)
	setXMLFloat(xmlFile, field_key .. "#limeFactor", self.limeFactor)
	setXMLFloat(xmlFile, field_key .. "#weedFactor", self.weedFactor)
	setXMLFloat(xmlFile, field_key .. "#stubbleFactor", self.stubbleFactor)

	setXMLString(xmlFile, field_key .. "#fruitTypeName", "GRASS")

	setXMLFloat(xmlFile, field_key .. "#areaToCut", self.area_tot)
	setXMLFloat(xmlFile, field_key .. "#areaTrimmed", self.area_trimmed)
	return true
end

function KommunalMission:init(field, ...)
	WeedMission:superClass().init(self, field, ...)
	self.fieldSpraySet = false
	local field_area = field.fieldArea
	-- si prende un'area media di 2.839, e si calcola da lì il valore dell'area da dare in base a quello
	local average_field_area = 0
	for field_idx=1, #g_fieldManager.fields do
		average_field_area = average_field_area + g_fieldManager.fields[field_idx].fieldArea
	end
	average_field_area = average_field_area / #g_fieldManager.fields

	self.area_tot = math.random()*400 + (1000 * field_area / average_field_area)
	self.rewardPerHa = 2*KommunalMission.REWARD_PER_HA * field_area/average_field_area
	self.field = field
	return true
end

-- non importa quale campo viene scelto per la missione, quindi lo accetto con una probabilità del 15% e finsice lì
function KommunalMission.canRunOnField(field, sprayFactor, fieldSpraySet, fieldPlowFactor, limeFactor, maxWeedState,
									   stubbleFactor, rollerFactor)
	return true
end

-- chiamato quando clicchi sul contratto per ottenere le informazioni
function KommunalMission:getData()
	local area_truncated = tonumber(string.format("%.0f", self.area_tot))

	return {
		location = "Anywhere",
		jobType = g_i18n:getText("fieldJob_jobType_kommunal"),
		action = g_i18n:getText("fieldJob_desc_action_kommunal"),
		description = string.format(g_i18n:getText("fieldJob_desc_kommunal"), area_truncated)
	}
end

function KommunalMission:getCompletion()
	return math.min(1, self.area_trimmed/self.area_tot)
end

function KommunalMission:validate(event)
	return true
end

function KommunalMission:updateRewardPerHa()
	return nil
end

function KommunalMission:getVehicleVariant()
	return nil
end

function KommunalMission:registerTrimmedArea(area)
	self.area_trimmed = self.area_trimmed + area
end

function KommunalMission:loadVehicles(xmlFilename, baseDirectory)
	local missionVehicles_fileId = loadXMLFile("MissionVehicles", xmlFilename)
	local mission_xml_key = "missionVehicles.mission"
	
	local groups = {}

	local group_idx = 0
	-- for each group
	while true do
		local group_xml_key = string.format("%s.group(%s)", mission_xml_key, group_idx)
		if not hasXMLProperty(missionVehicles_fileId, group_xml_key) then
			break
		end
		local field_size = Utils.getNoNil(getXMLString(missionVehicles_fileId, group_xml_key .. "#fieldSize"), "MEDIUM")
		local rewardScale = Utils.getNoNil(getXMLFloat(missionVehicles_fileId, group_xml_key .. "#rewardScale"), 1)

		local vehicles = {}
		local group = {
			rewardScale = rewardScale
		}

		local group_valid = true -- se un veicolo non è trovato, il gruppo non deve essere considerato valido
		local vehicle_idx = 0

		-- for each vehicle
		while true do
			local vehicle_xml_key = string.format("%s.vehicle(%s)", group_xml_key, vehicle_idx)
			if not hasXMLProperty(missionVehicles_fileId, vehicle_xml_key) then
				break
			end

			local mod_name = getXMLString(missionVehicles_fileId, vehicle_xml_key .. "#modName")
			local filename = getXMLString(missionVehicles_fileId, vehicle_xml_key .. "#filename")
			if filename ~= nil then
				if mod_name ~= nil then  -- si tratta di un veicolo in una mod
					filename = mod_name .. "/" .. filename
					filename = Utils.getFilename(filename, g_modsDirectory)
				else
					filename = Utils.getFilename(filename)
				end
			end

			local storeItem = g_storeManager:getItemByXMLFilename(filename)

			local configurations = {} -- da caricare
			local configuration_idx = 0
			while true do
				local conf_xml_Key = string.format("%s.configuration(%d)", vehicle_xml_key, configuration_idx)
				if not hasXMLProperty(missionVehicles_fileId, conf_xml_Key) then
					break
				end

				local conf_name = getXMLString(missionVehicles_fileId, conf_xml_Key .. "#name")
				local conf_id = getXMLInt(missionVehicles_fileId, conf_xml_Key .. "#id")

				if conf_name ~= nil and id ~= nil then
					configurations[conf_name] = conf_id
				end

				configuration_idx = configuration_idx + 1
			end
			
			if storeItem ~= nil then
				table.insert(vehicles, {
					filename = filename,
					configurations = configurations
				})
			else
				--Logging.error("%s: Unable to load store item for '%s'", xmlFilename, filename)
				group_valid = false
				break
			end

			vehicle_idx = vehicle_idx + 1
		end

		if groups[field_size] == nil then
			groups[field_size] = {}
		end

		if group_valid then
			group.vehicles = vehicles

			table.insert(groups[field_size], group)
			group.identifier = table.getn(groups[field_size])
		end


		group_idx = group_idx + 1
	end

	self.missionVehicles['kommunal'] = groups

	delete(missionVehicles_fileId) -- cancella handle al file
	return true
end

function KommunalMission:loadMissionVehicles(superFunc, xmlFilename, ...)
	local path_split = xmlFilename:split("/")
	-- se i veicoli sono quelli del file di questa mod, caricare questi, altrimenti ripassare alla funzione di default
	if #path_split >= 3 and path_split[#path_split-2] == modName then
		local path = Utils.getFilename(KommunalMission.mission_vehicles_xml, g_modsDirectory)
		return KommunalMission.loadVehicles(self, path, g_modsDirectory)		
	else
		return superFunc(self, xmlFilename, ...)
	end
end


function Contrastscheck()
	local bc_ModName = "FS22_BetterContracts"
	if g_modIsLoaded[bc_ModName] then
		Logging.error("FS22_KommunalMission is incompatible with BetterContracts! The mod will shut down.")
		local info_diag_data = {text = string.format(g_i18n:getText("better_contracts_conflict"))}
		g_gui:showInfoDialog(info_diag_data)
		return false
	end 
	return true
end

-- check BetterContracts sufficient version:
if Contrastscheck() then
	MissionManager.loadMissionVehicles = Utils.overwrittenFunction(MissionManager.loadMissionVehicles,
		KommunalMission.loadMissionVehicles)

	g_missionManager:registerMissionType(KommunalMission, "kommunal")
end